/*
***	Copyright:	1995-1999 Gnter Woigk
  				mailto:kio@myokay.net

 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

 	Permission to use, copy, modify, distribute, and sell this software and
 	its documentation for any purpose is hereby granted without fee, provided
 	that the above copyright notice appear in all copies and that both that
 	copyright notice and this permission notice appear in supporting
 	documentation, and that the name of the copyright holder not be used
 	in advertising or publicity pertaining to distribution of the software
 	without specific, written prior permission.  The copyright holder makes no
 	representations about the suitability of this software for any purpose.
 	It is provided "as is" without express or implied warranty.

 	THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 	EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 	CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 	TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 	PERFORMANCE OF THIS SOFTWARE.
*/


#ifndef ay_h
#define	ay_h	ay_h

#include "h/audio.h"


/* ==================================================
		sound generation
		hardware independent part
		ay and beeper emulation
=================================================== */
	
#define EAR_MASK	0x10			// speaker port bits
#define	MIC_MASK	0x08



// ----	type for sound output log ------------------------------------------------------
struct aylog
{	long	cycle;			// time stamp
	Char	reg;			// 0 ... 15 for ay, 16 for zx48 speaker
	Char	value;			// value written
};


/* ----	the sound channels of the ay chip ---------------------------------------------
*/
class Channel
{
// ----	the amplifier stage of a channel ----------------------------------------------
private:
	Sample	soundvolume;		// volume of this channel if not use envelope
	Sample	envelopevolume;		// volume output of envelope generator
	Sample	volume;			// sound or envelope
	Sample	output;			// volume or 0
	bool	use_envelope;
	bool	amp_in;

	void	AmpInput(bool);

public:
	void	SetEnvelopeVolume(Char);
	void	SetEnvelopeSample(Sample);
	void	UseEnvelope(bool);
	void	SetVolume(Char);
	Sample	GetOutput();

// ----	the mixer stage of a channel --------------------------------------------------
private:
	bool	channel_enable;		// application's additional per channel enable
	bool	sound_enable;		// from mixer control register
	bool	sound_in;		// from sound generator
	bool	sound_out;		// output of gate: (sound_in||!sound_enable)
	bool	noise_enable;		// from mixer control register
	bool	noise_in;		// from noise generator
	bool	noise_out;		// output of gate: (noise_in||!noise_enable)

	void	MixSoundInput(bool);

public:
	void	MixNoiseInput(bool);
	void	EnableChannel(bool);
	void	EnableSound(bool);
	void	EnableNoise(bool);

// ----	the sound generator stage of a channel ---------------------------------------
private:
	Long	divider;	// predivider for clock input
	Long	reload;		// reload value
	Char	fine,coarse;	// real calues for reload register
	Long	when;		// next tick to toggle state

public:	
		Channel(long clockdivider);
	void	Reset(long clockdivider);

	void	SetFinePeriod(Char);
	void	SetCoarsePeriod(Char);
	void	Trigger();
	long	When()		{ return (long)when; }
};


/* ----	the noise generator of the ay chip -------------------------------------------
*/
class Noise
{	Long	divider;	// predivider for clock input
	Long	reload;		// reload value
	Long	when;		// next tick to toggle output
	int	index;		// in duration[]
	bool	state;		// current output state: high/low
	Channel	*A,*B,*C;	// propagate output state
public:
		Noise(long clockdivider,Channel*,Channel*,Channel*);
	void	Reset(long clockdivider);
	void	SetPeriod(Char);
	void	Enable(long now, bool f=true);
	void	Trigger();
	long	When()		{ return when; }
};


/* ----	the envelope generator of the ay chip ----------------------------------------
*/
class Envelope
{	long	divider;	// predivider for clock input
	long	reload;		// reload value
	Char	fine,coarse;	// real calues for reload register
	long	when;		// next tick to inc/dec volume
	int	index;		// current output state: volume 0 ... 15
	bool	repeat;		// repeat ramping
	bool	invert;		// toggle ramping direction
	int	direction;	// ramping up/down/off
	Channel	*A,*B,*C;	// propagate output state
public:
		Envelope(long clockdivider, Channel*,Channel*,Channel*);
	void	Reset(long clockdivider);
	void	SetShape(long now,Char);
	void	SetFinePeriod(Char);
	void	SetCoarsePeriod(Char);
	void	Trigger();
	long	When()		{ return when; }
};


// ----	class for the beeper ---------------
class Beeper
{
	Char	enabled;	// ear and mic enabled mask
	Char	state;		// ear and mic pins
	Sample	sample;
public:
		Beeper();
	void	Reset();
	void	Enable(Char earmicmask);
	void	EnableEar(bool);
	void	EnableMic(bool);
	Sample	SetInput(Char);
	Sample	GetOutput();
};


// ----	class for ay sound chips ---------------
class ay
{
// sound device
	Dsp	dsp;

// AY sound chip registers:
	short	AYregnr;		// selected register
	Char	AYreg[16+1];	// 16 registers/ports of the AY chip
	Char	AYregMask[16];	// exising bits mask

// sound output log buffer for cpu:
	aylog	*sndlog;		// filled in by Do_Output()
	aylog	*sndlogptr;		// output log write pointer
	aylog	*sndlogend;		// output log end pointer

// clocks:
	long	ayclock;		// input clock of ay sound chip
	long	cpuclock;		// T cycle clock ((of z80 cpu))
	long	aycycles;		// cpu clocks per AY internal clocks
	long	dspcycles;		// cpu clocks per dsp sample

// subsample output:
	int		subsample;		// building the current sample
	long	subcycle;		// T cycle of start of build
	int		oldsample;		// value of last sample added to subsample
	long	oldcycle;		// T cycle of last sample added

// internal state:
	int	 	 enabled;		// which channels are enabled ?
	Beeper	 beeper;
	Noise	 N;
	Envelope E;
	Channel	 A,B,C;

	void	WriteSample	( long now, Sample s );
	void	SetRegister	( long now, int nr, Char c );

public:
		ay		( long cpuclock, long ayclock );    // creator
		~ay		( );	// destructor

	void	Reset		( long cpuclock, long ayclock );
	void	Play		( long now );
	void	Suspend		( );
	void	Resume		( long now );

	void	Enable		( int enabled );
	void	EnableAY	( bool f=true );
	void	EnableChannelA	( bool f=true );
	void	EnableChannelB	( bool f=true );
	void	EnableChannelC	( bool f=true );
	void	EnableBeeper	( bool f=true );
	void	EnableEarOutput	( bool f=true );
	void	EnableMicOutput	( bool f=true );

	int	DspState	( )		{ return dsp.State(); }


// select AY sound chip register
	void	SelectRegister	( int nr )	{ AYregnr = nr&0x0f; }

// read selected AY sound chip data register
	Char	ReadRegister	( )		{ return AYreg[AYregnr]&AYregMask[AYregnr]; }

// get arbitrary register without selecting it
	Char	GetRegister	( int nr )	{ return AYreg[nr&0x0f]; }

// set arbitrary AY sound chip data register without selecting it
	void	SetRegister	( int r,Char n ){ r &= 0x0f; AYreg[r] = n; }

// write to selected AY sound chip data register via sound log
	void	WriteRegister	( long cc, Char n )
	{
		AYreg[AYregnr] = n;
		if ( sndlogptr<sndlogend )
		{	sndlogptr->cycle = cc;
			sndlogptr->reg = AYregnr;
			sndlogptr->value = n;
			sndlogptr++;
		}
	}

// write beeper byte to sound log
	void	WriteBeeper	( long cc, Char n )
	{
		AYreg[16] = n;
		if ( sndlogptr<sndlogend )
		{	sndlogptr->cycle = cc;
			sndlogptr->reg = 16;
			sndlogptr->value = n;
			sndlogptr++;
		}
	}
};


#endif










