/*
***	Copyright:	1998-1999 Gnter Woigk
  				mailto:kio@myokay.net

 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

 	Permission to use, copy, modify, distribute, and sell this software and
 	its documentation for any purpose is hereby granted without fee, provided
 	that the above copyright notice appear in all copies and that both that
 	copyright notice and this permission notice appear in supporting
 	documentation, and that the name of the copyright holder not be used
 	in advertising or publicity pertaining to distribution of the software
 	without specific, written prior permission.  The copyright holder makes no
 	representations about the suitability of this software for any purpose.
 	It is provided "as is" without express or implied warranty.

 	THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 	EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 	CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 	TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 	PERFORMANCE OF THIS SOFTWARE.
*/


#ifndef keyboard_h
#define keyboard_h keyboard_h

#include <X11/Xlib.h>

/* ----	keyboard and joystick translation types:
	for kbd::Mapping(type)
*/
enum
{	kbdPhysical=0x10,	// prefer physical translation (from scan code)
	kbdLogical=0x20,	// use logical translation (from character code)
	kbdNoJoystick=0x01,	// no joystick attached
	kbdKempston,		// emulate Kempston interface
	kbdCursor,		// emulate cursor joystick	((not yet implemented))
	kbgIf2,			// emulate interface II		((not yet implemented))
	kbdAgf 			// emulate AGF interface	((not yet implemented))
};


/* ----	keyboard -------------------------
	handling of keyboard and joysticks
*/
struct kbd
{
	char	kbyte[8];	// io state of keyboard ports
	char	jbyte;		// io state of joystick port
	int	ktype;		// type of keyboard emulation
	int	jtype;		// type of joystick emulation

public:
		kbd		( )		{ Mapping(kbdLogical+kbdKempston); }
		kbd		( int type )	{ Mapping(type); }

	void	Mapping		( int type );			// select keyboard+joystick type
	void	Reset		( )		{ Mapping(0); }	// release all keys
	void	KeyEvent	( XKeyEvent *xev );		// process event
};



#endif
