
/*	KIO's standard header file.
	Can't live without it.
	Copyright (C) 1995-1999 G.Woigk
	kio@odn.de
	
	This file is free software
	See main.cc for details

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
*/

#ifndef	kio_h
#define	kio_h	kio_h


#include <stdlib.h>
#include <stdio.h>
#include <string.h>


// ---- Basic data type definitions ------------------------------------------
typedef	unsigned char		Char;
typedef	unsigned short		Short;
typedef	unsigned long		Long;
typedef unsigned int		Int;
typedef	long long			llong;
typedef	unsigned long long	LLong;
typedef	int					OSErr;
typedef unsigned char		*Ptr;
typedef char				*ptr;


// ----	Constants -----------------------------------------------------------
#define	true	1
#define	false 	0
#define	yes		1
#define	no		0
#define	on		1
#define	off		0
#define	nil		((void*)0)


// ----	Error codes ---------------------------------------------------------
enum
{	internalerror		= -2,		// unix: private error codes must be negative
	error			= -1,
	noerror			=  0,
	ok			=  0,
	success			=  0,		// standard unix error codes
	outofmemory		=  12,
	notanumber		=  34,
	numbertoobig		=  33,
	notimplemented		=  38,
	parametererror		=  22
};
extern	char*	errorText[];	// application's table with error messages starting with errorcode -1
extern	int	errorTextSize;	// size of errorText[]


/* ----	Global variables ----------------------------------------------------
*/
extern		bool	 	done;		// quit application ?	( 'apple'+'Q' )
extern		bool		abo;		// abort operation ?	( 'apple'+'.' )


// ----	Nice defines -------------------------------------------------------
#define		MIN(a,b)	((a)<(b)?(a):(b))
#define		MAX(a,b)	((a)>(b)?(a):(b))
#define		SWAP(a,b)	((a)^=(b),(b)^=(a),(a)^=(b))	//  (c) KIO !
#define		SIGN(a)		(((a)>0)-((a)<0))		//  (c) KIO !
#define		ABS(a)		((a)>0?(a):-(a))
#define		SORT(a,b)	if ((a)<=(b)) {} else SWAP(a,b)
#define		NELEM(array)	(sizeof(array)/sizeof(array[0]))
#define		BIT(B,N)	(((N)>>(B))&1)

// ----	Calculate base 2 logarithm -----------------------------------------
#define LOG2BYTE(N)		((N)>127+(N)>63+(N)>31+(N)>15+(N)>7+(N)>3+(N)>1)
#define LOG2SHORT(N)	(unsigned(N)>255 ? LOG2BYTE(unsigned(N)>>8)+8 : LOG2BYTE(N))
#define LOG2(N)			(unsigned(N)>0x7fff ? LOG2SHORT(unsigned(N)>>16)+16 : LOG2SHORT(N))

inline long	Min			( long a, long b )		{ return a<b ? a : b; }
inline long	Max			( long a, long b )		{ return a>b ? a : b; }	
inline long	MinMax		( long a, long n, long e )	{ return n<=e ? (n>=a?n:a) : e; }

inline int  Sign		( long a )		{ return SIGN(a); }
inline long	Abs			( long a )		{ return a>0 ? a : -a; }

inline void	Swap		( long& a, long& b )	{ long c=a; a=b; b=c; }
inline void	Swap		( void*& a, void*& b )	{ void*c=a; a=b; b=c; }
inline void	Sort		( long& a, long& b )	{ if (a>b) SWAP(a,b); }

inline Long	Scale		( Long oldval, Long oldmax, Long newmax )	// size conversion ((caveat: oldmax*newmax <= 0xFFFFFFFF))
							{ return ( oldval*newmax+oldmax/2 ) / oldmax; }

inline Long	Random		( Short n )   /* [0 ... n] */	{ return Scale ( (Short)random(), 0x0ffff, n ); }


// ----	kio-style block copy ------------------------------------------------
#include	<string.h>
#define	MemMove(a,b,c)	memmove(b,a,c)
#define	MemCopy(a,b,c)	memcpy(b,a,c)
#define	MemCmp(a,b,c)	memcmp(b,a,c)
#define	MemChr(a,b,c)	memchr(b,a,c)
#define	MemSet(a,b,c)	memset(b,a,c)
#define	MemClear(a,n)	memset(a,0,n)


// ----	alerts.cp -------------------------------------------------------------
#define	XSAFE		(SAFE>=1)
#define	XXSAFE		(SAFE>=2)
#define	XXXSAFE		(SAFE>=3)
#define	XLOG		(LOG>=1)
#define	XXLOG		(LOG>=2)
#define	XXXLOG		(LOG>=3)

char*	ErrorText	( OSErr err );
void	_trap_error_	( char* file, long line, OSErr );
#define	NIMP()		_trap_error_ ( __FILE__,__LINE__,notimplemented )
#define	IERR()		_trap_error_ ( __FILE__,__LINE__,internalerror )
#define	OMEM(XXX)	if ((XXX)) 	{} else _trap_error_ ( __FILE__,__LINE__,outofmemory )

#define	TRAP(XXX)	if ( !(XXX) )	{} else IERR()
#define	XTRAP(XXX)	if ( !XSAFE )	{} else TRAP(XXX)
#define	XXTRAP(XXX)	if ( !XXSAFE ) 	{} else TRAP(XXX)
#define	XXXTRAP(XXX)	if ( !XXXSAFE ) {} else TRAP(XXX)

void	LogNL		( );

void	Log			( const char* fmt, ... );
#define	XLog		if ( !XLOG )	{} else Log
#define	XXLog		if ( !XXLOG )	{} else Log
#define	XXXLog		if ( !XXXLOG )	{} else Log

#define	LogLine		LogNL(),Log
#define	XLogLine	if ( !XLOG )	{} else LogLine
#define	XXLogLine	if ( !XXLOG )	{} else LogLine
#define	XXXLogLine	if ( !XXXLOG )	{} else LogLine

#define	LogExit()	Log("..ok"),LogLine("<")
#define	XLogExit()	if ( !XLOG )	{} else LogExit()
#define	XXLogExit()	if ( !XXLOG )	{} else LogExit()
#define	XXXLogExit()	if ( !XXXLOG )	{} else LogExit()

#endif








