/*
 * Copyright (c) Des Herriott 1993, 1994
 * Copyright (c) Gnter Woigk 1999
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 * (and Thomas Ahn Kjaer for the banked port handling)
 * Gnter Woigk
 */


#ifndef		zxsp_window_h
#define		zxsp_window_h zxsp_window_h

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "kio.h"

class zxsp;

#define	WIN_COLORS	16
#define	WIN_GRID	16			// fit window dimensions to a multiple of 16
#define	WIN_MINWIDTH	(8*32)			// minimum width for window
#define WIN_MAXWIDTH	(8*32 *8)		// maximum ""
#define	WIN_MINHEIGHT	(8*24)			// minimum height for window
#define	WIN_MAXHEIGHT	(8*24 *8)		// maximum ""

#define	WIN_STDWIDTH	2*(WIN_MINWIDTH +4*8*2)	// default width
#define	WIN_STDHEIGHT	2*(WIN_MINHEIGHT+3*8*2)	// default height


class Win
{
	Display	*display;		// connection to X server
	Screen	*screen;
	Window	 root;
	Window	 mainWin;
	Visual	*visual;
	XVisualInfo visiinf;

	Colormap cmap;			// color map
	GC 	 gc;			// graphics context
	XImage	*xim;			// offscreen image

	Long 	 white, black;		// precalculated color patterns
	Long 	 color[WIN_COLORS];	// screenbytes for each color

	int	 xpos;			// current position of window
	int	 ypos;			// ""
	int 	 colordepth;		// color depth of xim in bits
	int	 pixelsize;		// pixel size = color depth + unused bits (e.g. 16 for 15bpp)
	int	 pixelmask;
	int	 visiclass;

	char	*name;			// window name
	Atom	 killme;		// hook for kill window event
	bool	 mitshm;		// use MIT SHared Memory ?
	bool	 fullupdate;		// flag set, when entire screen is updated
	char	*data;			// xim->data
	long	 yoffs;			// xim->bytes_per_line
	int	 scale;			// scaling factor: xim -> window
	int	 width;			// of window 
	int	 height;		// of window

	void	CreateColors	( );
	void	DestroyColors	( );
	void	CreateImage	( int width, int height );
	void	DestroyImage	( );

public:
		Win				( int argc, char* argv[] );
		~Win			( );

	void 	Update		( int x, int y, int width );			// invalidate scaled line
	void	FullUpdate	( )		{ fullupdate=true; }		// invalidate whole screen
	void	Sync		( );		// actually force drawing of updates receiced so far
	
	int	Yoffs			( )		{ return yoffs; }		// real xim line offset
	int	RealWidth		( )		{ return width; }		// unscaled width
	int	RealHeight		( )		{ return height; }		// unscaled height
	int	Width			( )		{ return xim->width/scale; }	// scaled width
	int	Height			( )		{ return xim->height; }		// scaled height
	int	Depth			( )		{ return pixelsize; }
	ptr	Data			( )		{ return data; }
	ptr	Data			( int y )	{ return data+y*yoffs; }
	ptr	Data			( int x, int y ){ return Data(y)+((x*scale*Depth())>>3); }

	char*	GetName		( )		{ return name; }
	void	SetName		( );

	Long	Color		( int i )	{ return color[i%WIN_COLORS]; };

	bool	Resized		( )		{ return xim==NULL; }
	void	SetScale	( int scale );

	void	PollEvents	( XEvent& ev );
	void	ConfigureEvent	( XEvent& ev );	// handle the event
	Display*Disp		( )		{ return display; }
	Atom	Killme		( )		{ return killme; }
};



#endif




