
/*
***	Copyright:	1998-1999 Gnter Woigk
  				mailto:kio@myokay.net

 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

 	Permission to use, copy, modify, distribute, and sell this software and
 	its documentation for any purpose is hereby granted without fee, provided
 	that the above copyright notice appear in all copies and that both that
 	copyright notice and this permission notice appear in supporting
 	documentation, and that the name of the copyright holder not be used
 	in advertising or publicity pertaining to distribution of the software
 	without specific, written prior permission.  The copyright holder makes no
 	representations about the suitability of this software for any purpose.
 	It is provided "as is" without express or implied warranty.

 	THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 	EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 	CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 	TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 	PERFORMANCE OF THIS SOFTWARE.
*/


#include 	<X11/Xlib.h>
#include 	<X11/Xutil.h>
#include 	<X11/keysym.h>

#include 	<stdio.h>
#include 	<string.h>

#include 	"h/options.h"
#include 	"h/kio.h"
#include 	"h/main.h"
#include 	"h/screen.h"

#ifndef 	LOG
#define 	LOG 1
#endif
#ifndef 	SAFE
#define 	SAFE 2
#endif

// definition is missing:
//#define AltMask	0x0008
//#define AltGrMask	0x2000


/* ----	there are 2 tables for the keyboard mapping:
		physical keycode -> specci keyboard descriptor
		character code -> specci keyboard descriptor

	specci keyboard descriptors are defined as follows:
		0x80	= add caps key
		0x08	= add symbol key
		0x00-70	= key's row
		0x00-04	= key's column
		0xFF	= not assigned
*/

/* ----	handy names for each specci key:
*/
enum
{	x=0xFF,		  			// unmapped
	c=0x80,					// add caps shift key
	s=0x08,					// add symbol shift key
	CSH=0x00,Z,X,C,V,		// left bottom row
	A=0x10,S,D,F,G,			// 	...
	Q=0x20,W,E,R,T,			//	...
	Z1=0x30,Z2,Z3,Z4,Z5,	// left top row
	Z0=0x40,Z9,Z8,Z7,Z6,	// right top row
	P=0x50,O,I,U,Y,			// 	...
	ENT=0x60,L,K,J,H,		//	...
	SPC=0x70,SSH,M,N,B		// right bottom row
};


/* ----	physical key codes translation:
*/
static Char phys2spec[] =	
{	x,x,x,x,x,x,x,x,x,c+s,	// 00-09	esc := caps+symbol shift
	Z1,Z2,Z3,Z4,Z5,			// 0a-0e	12345
	Z6,Z7,Z8,Z9,Z0,			// 0f-13	67890
	x,x,Z0+c,x,				// 14-17	delete
	Q,W,E,R,T,				// 18-1c	QWERT
	Y,U,I,O,P,				// 1d-21	YUIOP
	x,x,ENT,s,				// 22-25	enter, symbol shift
	A,S,D,F,G,				// 26-2a	ASDFG
	H,J,K,L,ENT,			// 2b-2f	HJKL, enter
	x,x,c,x,				// 30-33	caps shift
	Z,X,C,V,				// 34-37	ZXCV
	B,N,M,s,SPC,			// 38-3c	BNM, symbol shift, space
	x,c,B+s,				// 3d-3f	caps shift, [*]
	x,SPC,x,x,x,x,x,x,		// 40-47	space
	x,x,x,x,x,x,x,Z7,		// 48-4f	[7]
	Z8,Z9,J+s,Z4,			// 50-53	[8][9][-][4]
	Z5,Z6,K+s,Z1,			// 54-57	[5][6][+][1]
	Z2,Z3,Z0,M+s,x,x,c,x,	// 58-5f	[2][3][0][.], caps shift
	x,x,Z7+c,x,Z5+c,x,Z8+c,x,// 60-67	up, left, right
	Z6+c,x,x,x,ENT,s,x,x,	// 68-6f	down, [enter], symbol shift
	V+s,x,x,x,x,x,x,x,		// 70-77	[/]
};


/* ----	character codes translation:
*/
static Char char2spec[] =
{	x,x,x,x,x,x,x,x,
	x,x,x,x,x,x,x,x,
	x,x,x,x,x,x,x,x,
	x,x,x,x,x,x,x,x,

	SPC,Z1+s,P+s,Z3+s,Z4+s,Z5+s,Z6+s,Z7+s,	// space ! " # $ % & '
	Z8+s,Z9+s,B+s,K+s,N+s,J+s,M+s,V+s,		// ()*+,-./
	Z0,Z1,Z2,Z3,Z4,Z5,Z6,Z7,				// 01234567
	Z8,Z9,Z+s,O+s,R+s,L+s,T+s,C+s,			// 89:;<=>?
	Z2+s,A+c,B+c,C+c,D+c,E+c,F+c,G+c,		// @ABCDEFG
	H+c,I+c,J+c,K+c,L+c,M+c,N+c,O+c,		// HIJKLMNO
	P+c,Q+c,R+c,S+c,T+c,U+c,V+c,W+c,		// PQRSTUVW
	X+c,Y+c,Z+c,c+s+Y,c+s+D,c+s+U,H+s,Z0+s,	// XYZ[\]^_
	x,A,B,C,D,E,F,G,						// `abcdefg
	H,I,J,K,L,M,N,O,						// hijklmno
	P,Q,R,S,T,U,V,W,						// pqrstuvw
	X,Y,Z,F+c+s,S+c+s,G+c+s,A+c+s,0+s		// xyz{|}~<del>
};


static bool alt = false;	// state of alt key: additional shift key for 1...0


/* =======================================================================
			Keyboard Handler
======================================================================= */


/* ----	select new method for keystroke translation ---------------------
		valid values: see enum in "keyboard.h"
*/
void kbd::Mapping ( int type )
{
	if (type)	XLogLine ( "kbd::Mapping($%02X)", type );
	else		XXLogLine( "kbd::Reset()" );
		
// get new types:
	if (type&0xF0) ktype = type&0xF0;
	if (type&0x0F) jtype = type&0x0F;

	XXTRAP(!ktype);
	XXTRAP(!jtype);

// check types:
	if (ktype>kbdLogical)	ktype = kbdLogical;		// sanity check
	if (jtype>kbdKempston)	jtype = kbdKempston;	// others not yet implemented

// reset keyboard
	MemSet(0xff,kbyte,sizeof(kbyte));
	alt = false;

// reset joystick:
	switch(jtype)
	{
	case kbdNoJoystick:	jbyte = 0xFF;	break;
	case kbdKempston:	jbyte = 0x00;	break;
	}

	if (type&&XLOG)
	{
		static char* k[] = { "X", "physical", "logical" };
		static char* j[] = { "X", "no", "Kempston", "cursor", "IF2", "AGF" };
		Log ( "  => %s key map, %s joystick", k[ktype>>4], j[jtype] );
	}
}


/* --------------------------------------------------------------
	process a keydown or keyup event

	kbdType=kbdPhysical	=> physical mapping is preferred (for games)
	kbdType=kbdLogical	=> logical  mapping is preferred (for basic)

	the table of the preferred method is tried first. if it does not
	contain an entry for the pressed/released key, the other table is
	tried. if both don't match, a test for alt, meta and f-keys is made.

	the state of the alt and meta keys is monitored in static bool alt since there 
	is no #defined name for them in xev->state.
	they work as auxiliary "caps shift" keys in both mapping modes.
	they are essential in logical mapping mode to get the "caps shift" + "number key",
	e.g. "caps shift" + "1" to edit a basic line. if you simply press "shift" plus "1"
	this is "!" and this is decoded as "symbol shift" + "1" which results in a "!"
	on the specci. on the other hand, "alt" + "1" works. useful for breaking into 
	running basic programs too, since shift + space results in a simple space too.

	the state of control key is taken from xev->state in both modes, since it does
	not influence the character code returned by XLookupString().

	the state of shift and shift lock are taken from xev->state only in physical 
	mapping mode. they are not decoded separately in logical mode, since the key 
	translation table supplies the corresponding shift and control key state and an 
	additional modifier detected will disturb. e.g. if you press "!" (that is: "shift" + "1") 
	this results in "symbol shift" + "1" which is transposed to "!" by the specci
	keyboard routines just as expected. adding "caps shift" because "shift" is 
	pressed on the real keyboard (just to get the "!") will result in "1" and both
	shift keys pressed to the specci, decoding to nothing (3 keys pressed!)

---------------------------------------------------------------*/

void kbd::KeyEvent ( XKeyEvent *xev )
{
	bool down 		= xev->type == KeyPress;
	Long state		= xev->state;
	Long keycode	= xev->keycode;
	Char kspec		= x;
	Long charcode;

	XXLogLine 
	( 	"kbd::KeyEvent(xev): $%08X, $%08X %s", 
		xev->state, xev->keycode, down?"down":"up" 
	);

	if 
	(	ktype==kbdPhysical && 			// physical mapping preferred ?
		keycode<NELEM(phys2spec) && 	// inside table ?
		phys2spec[keycode]!=x			// mapping exists ?
	)
	{
		// physical mapping:
phys:	kspec = phys2spec[keycode];
		if ( down && (state&(ShiftMask|LockMask)) ) kspec |= c;
		goto doit;
	}
	
	// logical mapping
	// note: XLookupString(xev,0,0,&ks,0);  -> charcode->specci translation
	// note: ks=XLookupKeysym(xev,0);	-> config lookup for kempston joystick matching

	KeySym ks;
	XLookupString ( xev, NULL, 0, &ks, NULL );
	charcode = ks;
	XXLog("[%i]",charcode);
	
	if 
	(	charcode<=NELEM(char2spec) &&
		char2spec[charcode] != x
	)
	{	// logical mapping:
		kspec = char2spec[charcode];
		goto doit;
	}

	// we do not have a logical mapping
	// do we have a physical mapping instead ?
	if
	(	keycode<NELEM(phys2spec) && 	// inside physical table ?
		phys2spec[keycode]!=x		// mapping exists ?
	)
	{	// physical mapping should do it
		goto phys;
	}
	
	// neither physical nor logical map entry found
	// maybe it's an F-key:

	if (charcode>=XK_F1 && charcode<=XK_F12)
	{
		if (down) switch ( charcode ) 
		{
	//	case XK_F1:	specci.Reset(); break;
	//	case XK_F2:	specci.SetType( zx48+zx128 -specci.GetType() );	break;
		case XK_F3:	this->Mapping ( kbdLogical+kbdPhysical-ktype );	break;
	//	case XK_F4:	if (specci.soundchip.GetSoundOn()) specci.soundchip.Suspend(); else specci.soundchip.Resume(specci.Now()); break;
	//	case XK_F5:	break;
	//	case XK_F6:	break;
	//	case XK_F7:	break;
	//	case XK_F8:	break;
	//	case XK_F9:	break;
		case XK_F10:	done=true;	break;
	//	case XK_F11:	break;
	//	case XK_F12:	break;
		}
		return;
	}

	// last option: special characters and control codes	

	switch ( charcode )
	{
//	case XK_Return:		kspec=ENT;	break;
//	case XK_BackSpace:
//	case XK_Delete:		kspec=Z0+c;	break;
//	case XK_Escape:		kspec=c+s;	break;
//	case XK_Up:			kspec=Z7+c;	break;
//	case XK_Down:		kspec=Z6+c;	break;
//	case XK_Left:		kspec=Z5+c;	break;
//	case XK_Right:		kspec=Z8+c;	break;
	case XK_sterling:	kspec=X+s;	break;
//	case XK_Shift_L:
//	case XK_Shift_R:   	kspec=c;	break;
//	case XK_Control_L:
//	case XK_Control_R: 	kspec=s;	break;
	case XK_Meta_L:
	case XK_Meta_R:
	case XK_Alt_L:
	case XK_Alt_R:		alt=down;	return;
	default:
		if (!XXLOG)
		XLogLine 
		( 	"kbd::KeyEvent(xev): $%08X, $%08X %s", 
			xev->state, xev->keycode, down?"down":"up" 
		);
		XLog ( " ***unknown key*** XK-code: $%08X", charcode );

		return;	// no match !
	}

	
doit:
	XXTRAP(kspec==x);		// should never happen

	if (alt) kspec |= c;	// altL, altR, metaL, metaR are additional shift keys esp. for top row

	if ( down && (state&ControlMask) ) kspec |= s;	// control is not evaluated by the XLookupString()

	// symbol shift key:
	if (kspec&s)	kbyte[7] &= ~0x02;
	else			kbyte[7] |=  0x02;

	// caps shift key:
	if (kspec&c)	kbyte[0] &= ~0x01;
	else			kbyte[0] |=  0x01;

	if ( (kspec&~(c+s))==0 ) return;

	// the key itself
	char &b = kbyte[(kspec>>4)&7];		// affected io byte
	int   m = 1<<(kspec&7);				// affected bit mask
	if (down) b &= ~m; else b |= m;
}










